RangeMut

Struct RangeMut 

pub struct RangeMut<'gb, T: 'gb> { /* private fields */ }
Mutable sub-range of GapBuffer.

This struct is created by Slice::range_mut.

Implementations

Source

impl<'gb, T: 'gb> RangeMut<'gb, T>

pub const fn empty() -> Self

Construct a new, empty RangeMut.

Methods from Deref<Target = Slice<T>>

pub fn len(&self) -> usize

Returns the number of elements in the GapBuffer.

pub fn is_empty(&self) -> bool

Returns true if the GapBuffer contains no elements.

pub fn get(&self, index: usize) -> Option<&T>

Returns a reference to an element at index or None if out of bounds.

pub fn get_mut(&mut self, index: usize) -> Option<&mut T>

Returns a mutable reference to an element at index or None if out of bounds.

pub fn swap(&mut self, a: usize, b: usize)

Swaps two elements in the GapBuffer.

Arguments
  • a - The index of the first element
  • b - The index of the second element
Panics

Panics if a >= self.len() or b >= self.len().

pub fn range(&self, range: impl RangeBounds<usize>) -> Range<'_, T>

Return a immutable sub-range of this Slice.

Panics

Panics if range is out of bounds.

Examples
use gap_buf::gap_buffer;

let buf = gap_buffer![1, 2, 3, 4, 5];

let r1 = buf.range(1..);
assert_eq!(r1, [2, 3, 4, 5]);

let r2 = r1.range(1..3);
assert_eq!(r2, [3, 4]);
pub fn range_mut(&mut self, range: impl RangeBounds<usize>) -> RangeMut<'_, T>

Return a mutable sub-range of this Slice.

Panics

Panics if range is out of bounds.

Examples
use gap_buf::gap_buffer;

let mut buf = gap_buffer![1, 2, 3, 4, 5];
{
    let mut r = buf.range_mut(1..);
    assert_eq!(r, [2, 3, 4, 5]);
    r[0] = 0;
}
assert_eq!(buf, [1, 0, 3, 4, 5]);
pub fn as_slices(&self) -> (&[T], &[T])

Returns a pair of slices. First slice is before gap. Second slice is after gap.

Examples
use gap_buf::gap_buffer;

let mut buf = gap_buffer![1, 2, 3, 4, 5];
buf.set_gap(2);
let (s1, s2) = buf.as_slices();
assert_eq!(s1, [1, 2]);
assert_eq!(s2, [3, 4, 5]);
pub fn as_mut_slices(&mut self) -> (&mut [T], &mut [T])

Returns a pair of slices. First slice is before gap. Second slice is after gap.

Examples
use gap_buf::gap_buffer;

let mut buf = gap_buffer![1, 2, 3, 4, 5];
buf.set_gap(2);
{
    let (mut s1, mut s2) = buf.as_mut_slices();
    s1[0] = 10;
    s2[0] = 11;
}
assert_eq!(buf, [10, 2, 11, 4, 5]);
pub fn iter(&self) -> Iter<'_, T>

Returns an iterator over the Slice.

pub fn iter_mut(&mut self) -> IterMut<'_, T>

Returns an iterator that allows modifying each value.